#define M_PI 3.14159265359

#include <cmath>
#include <fstream>
#include <iostream>
#include <stdexcept> // std::runtime_error
#include <stdio.h>
#include <string>
#include <vector>

#include "Vehicle.h"
#include "sim.h"

#if defined(_WIN32) || defined(linux)
#include "native.h"
#elif TEENSY
#include "teensy.h"
#endif

Vehicle State;
Vehicle PrevState;
outVector stateVector;

#if defined(_WIN32) || defined(linux)
void setup() {
  init_Vehicle(State);

  // Determine when to burn
  burnStartTimeCalc(State);
}
#elif TEENSY
void setup() {
  init_Vehicle(State);

  // Determine when to burn
  burnStartTimeCalc(State);
  loadCellCalibrate();
}
#endif

void loop() {

  vehicleDynamics(State, PrevState);
  thrustInfo(State);
  pidController(State, PrevState);
  TVC(State, PrevState);
  processTVC(State);
  state2vec(State, PrevState, stateVector);

  State.time += State.stepSize;

  if (State.z < 0.0) {
    write2CSV(stateVector, State);
    printSimResults(State);
  }
}

#if defined(_WIN32) || defined(linux)
int main() {

  setup();

  do {
    loop();
  } while ((State.z > 0.0));

  return 0;
}
#endif